#!/usr/bin/env python
# coding: utf-8

"""
15.3 지도학습 모델의 종류
(p.263 ~ p.296)
"""

# In[1]: p.264
from sklearn.datasets import load_iris  # iris 데이터 세트 가져오기
iris = load_iris()  # iris 토이 데이터 세트를 iris변수에 저장
print(iris.feature_names)  # 특징(feature)들을 출력
print(iris.target_names)  # 학습 라벨에 해당하는 target을 출력
print(iris.data[0])  # 첫 번째 데이터에 저장된 feature값 출력
print(iris.target[0])  # 첫 번째 데이터에 저장된 target값 출력


# In[2]: p.265
from sklearn.datasets import load_iris
iris = load_iris()
for i in range(len(iris.target)):
    print("Example %d: label %s, features %s" % (i, iris.target[i], iris.data[i]))


# In[3]:
import numpy as np
from sklearn.datasets import load_iris
from sklearn import tree
iris = load_iris()
test = [0,50,100]

# training data 준비 과정
train_data = np.delete(iris.data, test, axis=0)  # 세 개의 데이터들을 데이터 변수에서 제거
train_target = np.delete(iris.target, test)  # 세 개의 데이터들을 타겟 변수에서 제거

# testing data 준비 과정
test_data = iris.data[test]
test_target = iris.target[test]

# 결정트리 생성
clf = tree.DecisionTreeClassifier()
clf.fit(train_data, train_target)

# 출력
print(test_target)
print(clf.predict(test_data))


# In[4]: p.267
a = np.array([[1,2], [3,4]])
print(a.flatten())  # 또는 print(a.flatten('C'))


# In[5]:
print(a.flatten('F'))


# In[6]:
import numpy as np
from sklearn.datasets import load_iris
from sklearn import tree
iris = load_iris()
test_idx = [0,50,100]
train_data = np.delete(iris.data, test_idx, axis=0)
print(train_data.shape)
print(train_data)


# In[7]:
import numpy as np
from sklearn.datasets import load_iris
from sklearn import tree
iris = load_iris()
test_idx = [0,50,100]
train_data = np.delete(iris.data, test_idx, axis=1)
print(train_data.shape)
print(train_data)


# In[8]: p.268
import numpy as np
from sklearn.datasets import load_iris
from sklearn import tree  # 구분자 훈련을 위한 import
iris = load_iris()
test_idx = [0,50,100]  # 3종류의 꽃 리스트에서 가장 앞에 있는 1개씩의 데이터 3개!
train_target = np.delete(iris.target, test_idx)  # 세 개의 요소들을 타겟 변수에서 제거
train_data = np.delete(iris.data, test_idx)  # 세 개의 요소들을 데이터 변수에서 제거
print(train_data.shape)
for i in range(len(train_data)):
    print("%dth, %s" % (i, train_data[i]))


# In[9]: p.269
from sklearn.datasets import load_iris
import matplotlib.pyplot as plt
iris = load_iris()
sepal = iris.data[:, 0:2]
kind = iris.target
plt.xlabel('Sepal Length')
plt.ylabel('Sepal Width')
plt.plot(sepal[kind==0][:,0], sepal[kind==0][:,1], "ro", label='Setosa')
plt.plot(sepal[kind==1][:,0], sepal[kind==1][:,1], "bo", label='Versicolor')
plt.plot(sepal[kind==2][:,0], sepal[kind==2][:,1], "yo", label='Virginica')
plt.legend()


# In[10]: p.270
import graphviz
dot_data = tree.export_graphviz(clf, out_file=None)
graph = graphviz.Source(dot_data)
graph.render("iris")
dot_data = tree.export_graphviz(clf, out_file=None,
                                feature_names=iris.feature_names,
                                class_names=iris.target_names,
                                filled=True, rounded=True,
                                special_characters=True)
graph = graphviz.Source(dot_data)
graph


# In[11]: p.271
from six import StringIO
import graphviz
dot_data = StringIO()
tree.export_graphviz(clf, out_file=dot_data,
                     feature_names = iris.feature_names,
                     class_names=iris.target_names,
                     filled=True, rounded=True,
                     impurity=False)
graph = graphviz.Source(dot_data.getvalue())
graph.render("iris", view=True)


# In[12]: p.272
print(test_data[1], test_target[1])
print(iris.feature_names, iris.target_names)


# In[13]:
import numpy as np
from sklearn.model_selection import train_test_split
y = range(6)
print(list(y))
print(train_test_split(y))


# In[14]: p.273
import numpy as np
from sklearn.model_selection import train_test_split
y = range(6)
print(list(y))
print(train_test_split(y, test_size=0.5))


# In[15]:
import numpy as np
from sklearn.model_selection import train_test_split
X = np.arange(12).reshape((6, 2))
y = range(6)
print(X)
print(list(y))


# In[16]:
import numpy as np
from sklearn.model_selection import train_test_split
X = np.arange(12).reshape((6, 2))
y = range(6)
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33)
print(X_train)
print(y_train)
print(X_test)
print(y_test)


# In[17]: p.274
from sklearn import datasets
iris = datasets.load_iris()
X = iris.data
y = iris.target
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=.5)
from sklearn import tree
clf = tree.DecisionTreeClassifier()
clf.fit(X_train, y_train)
predictions = clf.predict(X_test)
print('예측:\n', predictions)
print('정답(y_test):\n', y_test)


# In[18]: p.275
from sklearn.metrics import accuracy_score
a = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
b = [1, 2, 4, 4, 5, 6, 7, 8, 9, 10]
print(accuracy_score(a, b))


# In[19]:
from sklearn.metrics import accuracy_score
print(accuracy_score(y_test, predictions))


# In[20]: p.276
X = [[150], [156], [161], [167], [170], [174], [179]]
y = [51.1, 55.3, 59.4, 64.2, 66.5, 69.3, 74.2]
from sklearn.tree import DecisionTreeRegressor
regr = DecisionTreeRegressor()
regr.fit(X, y)
y_pred = regr.predict([[175]])
print(y_pred)


# In[21]:
from sklearn import tree.DecisionTreeRegressor
regr = tree.DecisionTreeRegressor()


# In[22]:
from sklearn.tree import DecisionTreeRegressor
regr = DecisionTreeRegressor()


# In[23]: p.278
import pandas as pd
data = pd.read_csv('acci.csv')
data.tail()


# In[24]: p.279
X = data.iloc[:, 3:]

y = data.iloc[:, 2]

X = data.iloc[:-1, 3:]
y = data.iloc[:-1, 2]

X = data.iloc[:, 3:]
X = X.values

X = data.iloc[:, 3:].values


# In[25]:
import pandas as pd
data = pd.read_csv('acci.csv')
X = data.iloc[:-1,3:]
y = data.iloc[:-1,2]
from sklearn.tree import DecisionTreeRegressor
regr = DecisionTreeRegressor()
regr.fit(X, y)
y_pred = regr.predict([[202, 11008, 3846, 6770, 392]])
print(y_pred)


# In[26]: p.280
import pandas as pd
data = pd.read_csv('acci.csv')
X = data[['발생건수', '부상자수', '중상', '경상']]
y = data['사망자수']
X = X.iloc[:-1,:]
y = y.iloc[:-1]
from sklearn.tree import DecisionTreeRegressor
regr = DecisionTreeRegressor()
regr.fit(X, y)
y_pred = regr.predict([[6193, 11008, 3846, 6770]])
print(y_pred)


# In[27]: p.281
X = [[0], [1], [2], [3], [4], [5]]
y = [0, 0, 0, 1, 1, 1]
from sklearn.neighbors import KNeighborsClassifier
neigh = KNeighborsClassifier(n_neighbors=3)
neigh.fit(X, y)


# In[28]: 
print(neigh.predict([[2.8]]))
print(neigh.predict_proba([[2.8]]))


# In[29]: p.282
print(neigh.predict([[2.3]]))
print(neigh.predict_proba([[2.3]]))


# In[30]:
from sklearn import datasets
iris = datasets.load_iris()
X = iris.data
y = iris.target
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=.5)

from sklearn.neighbors import KNeighborsClassifier
clf = KNeighborsClassifier()
clf.fit(X_train, y_train)
predictions = clf.predict(X_test)
print('예측:\n', predictions)
print('정답(y_test):\n', y_test)


# In[31]: p.283
from sklearn.metrics import accuracy_score
print(accuracy_score(y_test, predictions))


# In[32]: p.284
X = [[150], [156], [161], [167], [170], [174], [179]]
y = [51.1, 55.3, 59.4, 64.2, 66.5, 69.3, 74.2]
from sklearn.neighbors import KNeighborsRegressor
regr = KNeighborsRegressor(n_neighbors=3)
regr.fit(X, y)
y_pred = regr.predict([[175]])
print(y_pred)


# In[33]: 
import pandas as pd
data = pd.read_csv('acci.csv')
X = data.iloc[:-1,3:]
y = data.iloc[:-1,2]
from sklearn.neighbors import KNeighborsRegressor
regr = KNeighborsRegressor(n_neighbors=3)
regr.fit(X, y)
y_pred = regr.predict([[202, 11008, 3846, 6770, 392]])
print(y_pred)


# In[34]: p.285
import random
class myKNN():
    def fit(self, X_train, y_train):
        self.X_train = X_train  # X_train 데이터를 그대로 저장!
        self.y_train = y_train  # y_train 데이터를 그대로 저장!
    def predict(self, X_test):
        predictions = []
        for row in X_test:
            label = random.choice(self.y_train)
            predictions.append(label)
        return predictions

from sklearn import datasets
iris = datasets.load_iris()
X = iris.data
y = iris.target
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=.5)

# from sklearn.neighbors import KNeighborsClassifier 사용하지 않고
clf = myKNN()  # 나만의 분류자를 사용
clf.fit(X_train, y_train)
predictions = clf.predict(X_test)
from sklearn.metrics import accuracy_score
print(accuracy_score(y_test, predictions))


# In[35]: p.286
import numpy as np
def euc(p1, p2):
    return np.sqrt(np.sum(np.power(p2-p1, 2)))
p1 = np.array([1, 1])
p2 = np.array([4, 4])
euc(p1, p2)


# In[36]:
from scipy.spatial import distance
def euc(a,b) :
    return distance.euclidean(a, b)
p1 = np.array([1, 1])
p2 = np.array([4, 4])
euc(p1, p2)


# In[37]: p.287
from scipy.spatial import distance
def euc(a,b) :
    return distance.euclidean(a, b)
p1 = np.array([1, 1, 3, 3])
p2 = np.array([4, 4, 5, 5])
euc(p1, p2)


# In[38]: 
from scipy.spatial import distance
def euc(a,b) :
    return distance.euclidean(a, b)
class eucKNN():
    def fit(self, X_train, y_train):
        self.X_train = X_train
        self.y_train = y_train
    def predict(self, X_test):
        predictions = []
        for row in X_test:
            label = self.closest(row)
            predictions.append(label)
        return predictions
    def closest(self, row):
        best_dist = euc(row, self.X_train[0])
        best_index = 0
        for i in range(1, len(self.X_train)):
            dist = euc(row, self.X_train[i])
            if dist < best_dist:
                best_dist = dist
                best_index = i
        return self.y_train[best_index]
from sklearn import datasets
iris = datasets.load_iris()
X = iris.data
y = iris.target
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=.5)
clf = eucKNN()
clf.fit(X_train, y_train)
predictions = clf.predict(X_test)
from sklearn.metrics import accuracy_score
print(accuracy_score(y_test, predictions))


# In[39]: p.289
from sklearn import datasets
iris = datasets.load_iris()
X = iris.data
y = iris.target
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=.5)

from sklearn import svm
clf = svm.SVC(gamma=0.001, C=100.)
clf.fit(X_train, y_train)
predictions = clf.predict(X_test)
# print(y_test)
from sklearn.metrics import accuracy_score
print(accuracy_score(y_test, predictions))


# In[40]: p.290
X = [[150], [156], [161], [167], [170], [174], [179]]
y = [51.1, 55.3, 59.4, 64.2, 66.5, 69.3, 74.2]
from sklearn.svm import SVR
regr = SVR(gamma=0.001, C=100.)
regr.fit(X, y)
y_pred = regr.predict([[175]])
print(y_pred)


# In[41]: p.291
import pandas as pd
data = pd.read_csv('acci.csv')
X = data[['발생건수', '부상자수', '중상', '경상']]
y = data['사망자수']
X = X.iloc[:-1,:]
y = y.iloc[:-1]
from sklearn.svm import SVR
regr = SVR(gamma=0.001, C=100.)
regr.fit(X, y)
y_pred = regr.predict([[6193, 11008, 3846, 6770]])
print(y_pred)


# In[42]: p.292
loss = tf.reduce_mean(tf.square(y - y_data))


# In[43]: 
X = [[150], [156], [161], [167], [170], [174], [179]]
y = [51.1, 55.3, 59.4, 64.2, 66.5, 69.3, 74.2]
from sklearn.linear_model import LinearRegression
regr = LinearRegression()
regr.fit(X, y)
y_pred = regr.predict([[175]])
print(y_pred)


# In[44]: p.293
import pandas as pd
data = pd.read_csv('acci.csv')
X = data.iloc[:-1,3:]
y = data.iloc[:-1,2]
from sklearn.linear_model import LinearRegression
regr = LinearRegression()
regr.fit(X, y)
y_pred = regr.predict([[202, 11008, 3846, 6770, 392]])
print(y_pred)


# In[45]: p.294
""" [1] 케라스 버전을 위한 데이터 준비 """
import numpy as np
import matplotlib.pyplot as plt
points = 500
vectors = []
for i in range(points):
    x1= np.random.normal(0.0, 0.5)
    y1= x1 * 0.3 + 0.5 + np.random.normal(0.0, 0.05)
    vectors.append([x1, y1])
x_data = [v[0] for v in vectors]
y_data = [v[1] for v in vectors]
plt.scatter(x_data, y_data, color='yellow', label='TrainingData')


# In[46]: p.295
""" [2] 딥러닝 모델링 """
from tensorflow.keras import models
from tensorflow.keras import layers
model = models.Sequential()
model.add(layers.Dense(units=1, input_shape=[1]))
model.compile(loss='mse', optimizer='SGD') # ‘mean_squared_error'
history = model.fit(x_data, y_data, epochs=100, verbose=False)

""" [3] 경계선 그래프 """
weights, bias = model.get_weights()
result_line = x_data * weights[0] + bias
plt.scatter(x_data, y_data, color='yellow', label='TrainingData')
plt.plot(x_data, result_line, color='red', label='ResultLine')
plt.legend()
plt.show()


# In[47]: p.296
""" [4] 딥러닝 모델 테스트 """
test = np.array([[-1], [0], [1]])
pred = model.predict(test)
print(pred)

