#!/usr/bin/env python
# coding: utf-8

"""
16.4 이미지와 시계열 처리모델
<1> 이미지 처리모델
(p.353 ~ p.361)
"""

# In[1]: p.354
import numpy as np
from keras.utils import np_utils
from tensorflow.keras.datasets import mnist
(train_data, train_labels), (test_data, test_labels) = mnist.load_data()
print(test_labels)


# In[2]: p.355
import matplotlib.pyplot as plt
plt.imshow(test_data[0], cmap=plt.cm.YlOrBr)
print('Index %d: Label %d' % (0, test_labels[0]))


# In[3]:
print(train_data[0].shape)


# In[4]:
print(test_data[0])


# In[5]: p.357
width = 28
height = 28
x_train = train_data.reshape(60000, width*height).astype('float32') / 255.0
x_test = test_data.reshape(10000, width*height).astype('float32') / 255.0

# < 데이터 전처리 : one-hot 인코딩 >
y_train = np_utils.to_categorical(train_labels)
y_test = np_utils.to_categorical(test_labels)

# < 모델 구성 >
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense
model = Sequential()
model.add(Dense(256, input_dim=width*height, activation='relu'))
model.add(Dense(256, activation='relu'))
model.add(Dense(10, activation='softmax'))

# < 모델 설정 >
model.compile(loss='categorical_crossentropy', optimizer='sgd', metrics=['acc'])

# < 모델 학습 >
history = model.fit(x_train, y_train, epochs=20, batch_size=32)
model.evaluate(x_test, y_test, batch_size=32)


# In[6]: p.358
from keras.utils import np_utils
from tensorflow.keras.datasets import mnist
(train_data, train_labels), (test_data, test_labels) = mnist.load_data()
width = 28
height = 28
train_data = train_data.reshape(60000, width*height).astype('float32') / 255.0
test_data = test_data.reshape(10000, width*height).astype('float32') / 255.0


# In[7]:
from sklearn import tree
clf = tree.DecisionTreeClassifier()
clf.fit(train_data, train_labels)
predictions = clf.predict(test_data)
from sklearn.metrics import accuracy_score
print(accuracy_score(test_labels, predictions))


# In[8]: p.359
from sklearn.neighbors import KNeighborsClassifier
from sklearn import datasets
digits = datasets.load_digits()
clf = KNeighborsClassifier()
clf.fit(train_data, train_labels)
predictions = clf.predict(test_data)
from sklearn.metrics import accuracy_score
print(accuracy_score(test_labels, predictions))


# In[9]:
import random
class myKNN():
    def fit(self, train_data, train_labels):
        self.train_data = train_data
        self.train_labels = train_labels
    def predict(self, test_data):
        predictions = []
        for row in test_data:
            label = random.choice(self.train_labels)
            predictions.append(label)
        return predictions
clf = myKNN()  # 나만의 구분자를 사용
clf.fit(train_data, train_labels)
predictions = clf.predict(test_data)
from sklearn.metrics import accuracy_score
print(accuracy_score(test_labels, predictions))


# In[10]: p.360
from scipy.spatial import distance
def euc(a,b) :
    return distance.euclidean(a, b)
class eucKNN():
    def fit(self, train_data, train_labels):
        self.train_data = train_data
        self.train_labels = train_labels
    def predict(self, test_data):
        predictions = []
        for row in test_data:
            label = self.closest(row)
            predictions.append(label)
        return predictions
    def closest(self, row):
        best_dist = euc(row, self.train_data[0])
        best_index = 0
        for i in range(1, len(self.train_data)):
            dist = euc(row, self.train_data[i])
            if dist < best_dist:
                best_dist = dist
                best_index = i
        return self.train_labels[best_index]
clf = eucKNN()
clf.fit(train_data, train_labels)
predictions = clf.predict(test_data)
from sklearn.metrics import accuracy_score
print(accuracy_score(test_labels, predictions))


# In[11]:
from sklearn import datasets
digits = datasets.load_digits()
print(len(digits.data), len(digits.target))


# In[12]: p.361
from scipy.spatial import distance
def euc(a,b) :
    return distance.euclidean(a, b)
class eucKNN():
    def fit(self, X_train, y_train):
        self.X_train = X_train
        self.y_train = y_train
    def predict(self, X_test):
        predictions = []
        for row in X_test:
            label = self.closest(row)
            predictions.append(label)
        return predictions
    def closest(self, row):
        best_dist = euc(row, self.X_train[0])
        best_index = 0
        for i in range(1, len(self.X_train)):
            dist = euc(row, self.X_train[i])
            if dist < best_dist:
                best_dist = dist
                best_index = i
        return self.y_train[best_index]
clf = eucKNN()
clf.fit(digits.data[:-500], digits.target[:-500])
predictions = clf.predict(digits.data[-500:])
from sklearn.metrics import accuracy_score
print(accuracy_score(digits.target[-500:], predictions))


# In[13]:
from sklearn import svm
clf = svm.SVC(gamma=0.001, C=100.)
clf.fit(train_data, train_labels)
predictions = clf.predict(test_data)
from sklearn.metrics import accuracy_score
print(accuracy_score(test_labels, predictions))

